{$P+}
{$L+}

PROGRAM HARMONOGRAPH;
{Version 19}

{
Aim - To draw a diminishing circle superimposed on a 
      rotating diminishing ellipse.

Variable name terminology:-

   ellip, circ = ellipse, circle.
   len, wid    = ellipse length and width.
   rad = circle radius.
   ang = angle.
   axisang = axis angle for rotation.
   incr = increment (-ve = decrement).
   start = starting value.

   e.g  elliplenincr = increment for ellipse length
   to be added at each calculation/plot.





{Switch off run time checks for speed}
{$O-,S-,A-}



{-----------------------------------------------------}
{Constants}   
{-----------------------------------------------------}
CONST
   {Offset from origin}
   xoffset = 128;
   yoffset = 87;


   {Dimension start values}
   elliplenstart = 120;
   ellipwidstart = 20;
   circradstart  = 5;


   {Angle start values}
   ellipangstart = 1.5;
   circangstart  = 1.5;
   axisangstart  = 0;

   
   {Dimension increments}
   elliplenincr  = -0.08;
   ellipwidincr  = -0.01;
   circradincr   = -0.001;


   {Angle increments}
   ellipangincr  = -0.3;
   circangincr   = 0.32;
   axisangincr   = -0.003;


   {Dimension limits}
   elliplenlimit = 20;
   ellipwidlimit = 5;
   circradlimit  = 1;

   {Angle limits (if reqd}
   axisanglimit = -3; {radians}


{-----------------------------------------------------}
{Variables}   
{-----------------------------------------------------}
VAR
   {Dimensions}
   elliplen,
   ellipwid,
   circrad   : REAL;

   {Angles}
   ellipang,
   circang,
   axisang   : REAL;

   {Coordinates}
   xold,
   xnew,
   yold,
   ynew      : INTEGER;



{-----------------------------------------------------}
{Graphics Support Routines (from HISOFT manual}
{-----------------------------------------------------}

PROCEDURE drawlinesupport
          (x, y, signx, signy : INTEGER);

{
Aim - used in conjunction with the LINEDRAW procedure.
      Machine code sets up Z80 regs &
      calls Spectrum ROM DRAW routine.
}

BEGIN

INLINE
(
#FD,#21,#3A,#5C,  {LD IY,#5C3A}
#DD,#56,#02,      {LD D,(IX+2)}
#DD,#5E,#04,      {LD E,(IX+4)}
#DD,#46,#06,      {LD B,(IX+6)}
#DD,#4E,#08,      {LD C,(IX+8)}
#CD,#BA,#24       {CALL #24BA ;ROM DRAW routine}
)

END;
{-----------------------------------------------------}

PROCEDURE drawline (x, y : INTEGER);

{
Aim - To draw a line from the current plot position
      (CX, CY) to (CX+x, CY+y).
      (Equivalent to BASIC DRAW command).
}

VAR
   signx,
   signy  : INTEGER;


BEGIN

{Calculate sign of x & y}
IF x < 0 THEN
   signx := -1
ELSE
   signx := 1;

IF y < 0 THEN
   signy := -1
ELSE
   signy := 1;

drawlinesupport(ABS (x), ABS (y), signx, signy)

END;
{-----------------------------------------------------}

PROCEDURE plot (x, y : INTEGER);

{
Aim - To plot the specified point (x,y).
      Machine code sets up Z80 regs & 
      calls Spectrum ROM PLOT routine.
      (Equivalent to BASIC PLOT command).
}


BEGIN

INLINE
(
#FD,#21,#3A,#5C,  {LD IY,#5C3A}
#DD,#46,#02,      {LD B,(IX+2)}
#DD,#4E,#04,      {LD C,(IX+2)}
#CD,#E5,#22       {CALL #22E5  ;ROM PLOT routine}
)

END;
{-----------------------------------------------------}





{-----------------------------------------------------}
{Harmonograph Routines}
{-----------------------------------------------------}

FUNCTION xcalc : INTEGER;

{
Aim - To calculate & return the new value of X.
}

BEGIN

xcalc := ENTIER (
         elliplen * COS (ellipang) * COS (axisang) +
         ellipwid * SIN (ellipang) * SIN (axisang) +
         circrad * COS (circang) +
         xoffset)

END;
{-----------------------------------------------------}

FUNCTION ycalc : INTEGER;

{
Aim - To calculate & return the new value of Y.
}

BEGIN

ycalc := ENTIER (
         ellipwid * SIN (ellipang) * COS (axisang) +
         elliplen * COS (ellipang) * SIN (axisang) +
         circrad * SIN (circang) +
         yoffset)

END;
{-----------------------------------------------------}

PROCEDURE initialisesizes;

{
Aim - To initialise all dimension & angle sizes
      to their start values as defined in the CONST
      section.
}


BEGIN

elliplen := elliplenstart;
ellipwid := ellipwidstart;
circrad := circradstart;


ellipang := ellipangstart;
circang := circangstart;
axisang := axisangstart;

END;
{-----------------------------------------------------}

PROCEDURE incrementsizes;

{
Aim - To increment dimension & angle sizes by the
      values defined in the CONST section.
      Sizes can be decreased by using -ve increments.
}

BEGIN

elliplen := elliplen + elliplenincr;
ellipwid := ellipwid + ellipwidincr;
circrad := circrad + circradincr;

ellipang := ellipang + ellipangincr;
circang := circang + circangincr;
axisang := axisang + axisangincr;

END;
{-----------------------------------------------------}


{-----------------------------------------------------}
{MAIN PROGRAM BODY}
{-----------------------------------------------------}

BEGIN

WRITE (CHR (12)); {clear screen}

initialisesizes;

{Calculate & plot 1st point as a reference}
xold := xcalc;
yold := ycalc;
plot (xold, yold);

   WHILE
   {
   Loop until limits reached.
   'Comment-out' checks as reqd. for speed.
   }
         (axisang > axisanglimit)
   {
         AND
         (elliplen > elliplenlimit)
         AND
         (ellipwid > ellipwidlimit)
         AND
         (circrad > circradlimit)
   }
   DO
   BEGIN

   incrementsizes;

   {Calculate new point}
   xnew := xcalc;
   ynew := ycalc;

   {Draw line from old point to new}
   drawline (xnew - xold, ynew - yold);

   {Transfer new point to old for next time}
   xold := xnew;
   yold := ynew;

   END;


{
Loop-stop to prevent PASCAL prompt from racking up 
the picture & losing the top line!
Can break in using SHIFT+SPACE as normal.
}
WHILE TRUE DO;

END.



